@@ -3,6 +3,7 @@ Implement the command-line tool interface for diff_quality.
 """
 
 import argparse
+import io
 import logging
 import os
 import sys
@@ -11,40 +12,48 @@ import pluggy
 
 import diff_cover
 from diff_cover import hookspecs
+from diff_cover.config_parser import Tool, get_config
 from diff_cover.diff_cover_tool import (
     COMPARE_BRANCH_HELP,
+    CONFIG_FILE_HELP,
+    CSS_FILE_HELP,
     DIFF_RANGE_NOTATION_HELP,
+    EXCLUDE_HELP,
     FAIL_UNDER_HELP,
+    HTML_REPORT_HELP,
     IGNORE_STAGED_HELP,
     IGNORE_UNSTAGED_HELP,
-    EXCLUDE_HELP,
-    HTML_REPORT_HELP,
-    CSS_FILE_HELP,
     IGNORE_WHITESPACE,
+    INCLUDE_UNTRACKED_HELP,
+    JSON_REPORT_HELP,
+    MARKDOWN_REPORT_HELP,
+    QUIET_HELP,
 )
 from diff_cover.diff_reporter import GitDiffReporter
 from diff_cover.git_diff import GitDiffTool
 from diff_cover.git_path import GitPathTool
 from diff_cover.report_generator import (
     HtmlQualityReportGenerator,
+    JsonReportGenerator,
+    MarkdownQualityReportGenerator,
     StringQualityReportGenerator,
 )
 from diff_cover.violationsreporters.base import QualityReporter
-from diff_cover.violationsreporters.violations_reporter import (
-    CppcheckDriver,
-    flake8_driver,
-    pyflakes_driver,
-    PylintDriver,
-    jshint_driver,
-    eslint_driver,
-    pydocstyle_driver,
-    pycodestyle_driver,
-)
 from diff_cover.violationsreporters.java_violations_reporter import (
     CheckstyleXmlDriver,
-    checkstyle_driver,
     FindbugsXmlDriver,
     PmdXmlDriver,
+    checkstyle_driver,
+)
+from diff_cover.violationsreporters.violations_reporter import (
+    CppcheckDriver,
+    EslintDriver,
+    PylintDriver,
+    flake8_driver,
+    jshint_driver,
+    pycodestyle_driver,
+    pydocstyle_driver,
+    pyflakes_driver,
 )
 
 QUALITY_DRIVERS = {
@@ -54,7 +63,7 @@ QUALITY_DRIVERS = {
     "pylint": PylintDriver(),
     "flake8": flake8_driver,
     "jshint": jshint_driver,
-    "eslint": eslint_driver,
+    "eslint": EslintDriver(),
     "pydocstyle": pydocstyle_driver,
     "checkstyle": checkstyle_driver,
     "checkstylexml": CheckstyleXmlDriver(),
@@ -67,6 +76,8 @@ VIOLATION_CMD_HELP = "Which code quality tool to use (%s)" % "/".join(
 )
 INPUT_REPORTS_HELP = "Which violations reports to use"
 OPTIONS_HELP = "Options to be passed to the violations tool"
+INCLUDE_HELP = "Files to include (glob pattern)"
+REPORT_ROOT_PATH_HELP = "The root path used to generate a report"
 
 
 LOGGER = logging.getLogger(__name__)
@@ -95,15 +106,27 @@ def parse_quality_args(argv):
         "--html-report",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=HTML_REPORT_HELP,
     )
 
+    parser.add_argument(
+        "--json-report",
+        metavar="FILENAME",
+        type=str,
+        help=JSON_REPORT_HELP,
+    )
+
+    parser.add_argument(
+        "--markdown-report",
+        metavar="FILENAME",
+        type=str,
+        help=MARKDOWN_REPORT_HELP,
+    )
+
     parser.add_argument(
         "--external-css-file",
         metavar="FILENAME",
         type=str,
-        default=None,
         help=CSS_FILE_HELP,
     )
 
@@ -111,70 +134,106 @@ def parse_quality_args(argv):
         "--compare-branch",
         metavar="BRANCH",
         type=str,
-        default="origin/master",
         help=COMPARE_BRANCH_HELP,
     )
 
+    parser.add_argument("input_reports", type=str, nargs="*", help=INPUT_REPORTS_HELP)
+
+    parser.add_argument("--options", type=str, nargs="?", help=OPTIONS_HELP)
+
     parser.add_argument(
-        "input_reports", type=str, nargs="*", default=[], help=INPUT_REPORTS_HELP
+        "--fail-under", metavar="SCORE", type=float, help=FAIL_UNDER_HELP
     )
 
     parser.add_argument(
-        "--options", type=str, nargs="?", default=None, help=OPTIONS_HELP
-    )
-
-    parser.add_argument(
-        "--fail-under", metavar="SCORE", type=float, default="0", help=FAIL_UNDER_HELP
-    )
-
-    parser.add_argument(
-        "--ignore-staged", action="store_true", default=False, help=IGNORE_STAGED_HELP
+        "--ignore-staged", action="store_true", default=None, help=IGNORE_STAGED_HELP
     )
 
     parser.add_argument(
         "--ignore-unstaged",
         action="store_true",
-        default=False,
+        default=None,
         help=IGNORE_UNSTAGED_HELP,
     )
 
+    parser.add_argument(
+        "--include-untracked",
+        action="store_true",
+        default=None,
+        help=INCLUDE_UNTRACKED_HELP,
+    )
+
     parser.add_argument(
         "--exclude", metavar="EXCLUDE", type=str, nargs="+", help=EXCLUDE_HELP
     )
 
+    parser.add_argument(
+        "--include", metavar="INCLUDE", nargs="+", type=str, help=INCLUDE_HELP
+    )
+
     parser.add_argument(
         "--diff-range-notation",
         metavar="RANGE_NOTATION",
         type=str,
-        default="...",
         help=DIFF_RANGE_NOTATION_HELP,
     )
 
     parser.add_argument(
         "--version",
         action="version",
-        version="diff-quality {}".format(diff_cover.VERSION),
+        version=f"diff-quality {diff_cover.VERSION}",
     )
     parser.add_argument(
         "--ignore-whitespace",
         action="store_true",
-        default=False,
+        default=None,
         help=IGNORE_WHITESPACE,
     )
 
-    return vars(parser.parse_args(argv))
+    parser.add_argument(
+        "-q", "--quiet", action="store_true", default=None, help=QUIET_HELP
+    )
+
+    parser.add_argument(
+        "-c", "--config-file", help=CONFIG_FILE_HELP, metavar="CONFIG_FILE"
+    )
+
+    parser.add_argument(
+        "--report-root-path", help=REPORT_ROOT_PATH_HELP, metavar="ROOT_PATH"
+    )
+
+    defaults = {
+        "ignore_whitespace": False,
+        "compare_branch": "origin/main",
+        "diff_range_notation": "...",
+        "input_reports": [],
+        "fail_under": 0,
+        "ignore_staged": False,
+        "ignore_unstaged": False,
+        "ignore_untracked": False,
+        "quiet": False,
+    }
+
+    return get_config(
+        parser=parser, argv=argv, defaults=defaults, tool=Tool.DIFF_QUALITY
+    )
 
 
 def generate_quality_report(
     tool,
     compare_branch,
     html_report=None,
+    json_report=None,
+    markdown_report=None,
     css_file=None,
     ignore_staged=False,
     ignore_unstaged=False,
+    include_untracked=False,
     exclude=None,
+    include=None,
     diff_range_notation=None,
     ignore_whitespace=False,
+    quiet=False,
 ):
     """
     Generate the quality report, using kwargs from `parse_args()`.
@@ -187,8 +246,10 @@ def generate_quality_report(
         git_diff=GitDiffTool(diff_range_notation, ignore_whitespace),
         ignore_staged=ignore_staged,
         ignore_unstaged=ignore_unstaged,
+        include_untracked=include_untracked,
         supported_extensions=supported_extensions,
         exclude=exclude,
+        include=include,
     )
 
     if html_report is not None:
@@ -202,26 +263,40 @@ def generate_quality_report(
             with open(css_file, "wb") as output_file:
                 reporter.generate_css(output_file)
 
+    if json_report is not None:
+        reporter = JsonReportGenerator(tool, diff)
+        with open(json_report, "wb") as output_file:
+            reporter.generate_report(output_file)
+
+    if markdown_report is not None:
+        reporter = MarkdownQualityReportGenerator(tool, diff)
+        with open(markdown_report, "wb") as output_file:
+            reporter.generate_report(output_file)
+
     # Generate the report for stdout
     reporter = StringQualityReportGenerator(tool, diff)
-    output_file = sys.stdout.buffer
-
+    output_file = io.BytesIO() if quiet else sys.stdout.buffer
     reporter.generate_report(output_file)
+
     return reporter.total_percent_covered()
 
 
 def main(argv=None, directory=None):
     """
-    Main entry point for the tool, used by setup.py
+    Main entry point for the tool, script installed via pyproject.toml
     Returns a value that can be passed into exit() specifying
     the exit code.
     1 is an error
     0 is successful run
     """
-    logging.basicConfig(format="%(message)s")
 
     argv = argv or sys.argv
     arg_dict = parse_quality_args(argv[1:])
+
+    quiet = arg_dict["quiet"]
+    level = logging.ERROR if quiet else logging.WARNING
+    logging.basicConfig(format="%(message)s", level=level)
+
     GitPathTool.set_cwd(directory)
     fail_under = arg_dict.get("fail_under")
     tool = arg_dict["violations"]
@@ -237,10 +312,14 @@ def main(argv=None, directory=None):
     if driver is None:
         # The requested tool is not built into diff_cover. See if another Python
         # package provides it.
-        pm = pluggy.PluginManager("diff_cover")
-        pm.add_hookspecs(hookspecs)
-        pm.load_setuptools_entrypoints("diff_cover")
-        for hookimpl in pm.hook.diff_cover_report_quality.get_hookimpls():
+        plugin_manager = pluggy.PluginManager("diff_cover")
+        plugin_manager.add_hookspecs(hookspecs)
+        plugin_manager.load_setuptools_entrypoints("diff_cover")
+
+        hooks = (
+            plugin_manager.hook.diff_cover_report_quality  # pylint: disable=no-member
+        )
+        for hookimpl in hooks.get_hookimpls():
             if hookimpl.plugin_name == tool:
                 reporter = hookimpl.function()
                 break
@@ -256,31 +335,43 @@ def main(argv=None, directory=None):
                     try:
                         input_reports.append(open(path, "rb"))
                     except OSError:
-                        LOGGER.warning("Could not load '{}'".format(path))
+                        LOGGER.error("Could not load report '%s'", path)
+                        return 1
+
+                if arg_dict["report_root_path"]:
+                    driver.add_driver_args(
+                        report_root_path=arg_dict["report_root_path"]
+                    )
+
                 reporter = QualityReporter(driver, input_reports, user_options)
 
             percent_passing = generate_quality_report(
                 reporter,
                 arg_dict["compare_branch"],
                 html_report=arg_dict["html_report"],
+                json_report=arg_dict["json_report"],
+                markdown_report=arg_dict["markdown_report"],
                 css_file=arg_dict["external_css_file"],
                 ignore_staged=arg_dict["ignore_staged"],
                 ignore_unstaged=arg_dict["ignore_unstaged"],
+                include_untracked=arg_dict["include_untracked"],
                 exclude=arg_dict["exclude"],
+                include=arg_dict["include"],
                 diff_range_notation=arg_dict["diff_range_notation"],
                 ignore_whitespace=arg_dict["ignore_whitespace"],
+                quiet=quiet,
             )
             if percent_passing >= fail_under:
                 return 0
-            else:
-                LOGGER.error("Failure. Quality is below {}%.".format(fail_under))
-                return 1
+
+            LOGGER.error("Failure. Quality is below %i.", fail_under)
+            return 1
 
         except ImportError:
-            LOGGER.error("Quality tool not installed: '{}'".format(tool))
+            LOGGER.error("Quality tool not installed: '%s'", tool)
             return 1
         except OSError as exc:
-            LOGGER.error("Failure: '{}'".format(exc))
+            LOGGER.error("Failure: '%s'", str(exc))
             return 1
         # Close any reports we opened
         finally:
@@ -288,7 +379,7 @@ def main(argv=None, directory=None):
                 file_handle.close()
 
     else:
-        LOGGER.error("Quality tool not recognized: '{}'".format(tool))
+        LOGGER.error("Quality tool not recognized: '%s'", tool)
         return 1
 
 
