@@ -1,28 +1,31 @@
 """
 Classes for querying which lines have changed based on a diff.
 """
-from abc import ABCMeta, abstractmethod
-from diff_cover.git_diff import GitDiffError
 import fnmatch
+import glob
 import os
 import re
+from abc import ABC, abstractmethod
+
+from diff_cover.git_diff import GitDiffError
 
 
-class BaseDiffReporter:
+class BaseDiffReporter(ABC):
     """
     Query information about lines changed in a diff.
     """
 
-    __metaclass__ = ABCMeta
     _exclude = None
+    _include = None
 
-    def __init__(self, name, exclude=None):
+    def __init__(self, name, exclude=None, include=None):
         """
         Provide a `name` for the diff report, which will
         be included in the diff coverage report.
         """
         self._name = name
         self._exclude = exclude
+        self._include = include
 
     @abstractmethod
     def src_paths_changed(self):
@@ -31,7 +34,6 @@ class BaseDiffReporter:
 
         Source paths are guaranteed to be unique.
         """
-        pass
 
     @abstractmethod
     def lines_changed(self, src_path):
@@ -42,7 +44,6 @@ class BaseDiffReporter:
         Each line is guaranteed to be included only once in the list
         and in ascending order.
         """
-        pass
 
     def name(self):
         """
@@ -72,12 +73,22 @@ class BaseDiffReporter:
         """
         Check if a path is excluded.
 
+        First it is checked if the path matches one of the include patterns (if provided).
+        Second, the path is matched against the exclude patterns.
+
         :param str path:
-            Path to check against the exclude patterns.
+            Path to check against the exclude and include patterns.
         :returns:
-            True if there are exclude patterns and the path matches,
-            otherwise False.
+            True if the patch should be excluded, otherwise False.
         """
+        include = self._include
+        if include:
+            for pattern in include:
+                if path in glob.glob(pattern, recursive=True):
+                    break  # file is included
+            else:
+                return True
+
         exclude = self._exclude
         if not exclude:
             return False
@@ -86,8 +97,7 @@ class BaseDiffReporter:
             return True
 
         absolute_path = os.path.abspath(path)
-        match = self._fnmatch(absolute_path, exclude)
-        return match
+        return self._fnmatch(absolute_path, exclude)
 
 
 class GitDiffReporter(BaseDiffReporter):
@@ -97,26 +107,30 @@ class GitDiffReporter(BaseDiffReporter):
 
     def __init__(
         self,
-        compare_branch="origin/master",
+        compare_branch="origin/main",
         git_diff=None,
         ignore_staged=None,
         ignore_unstaged=None,
+        include_untracked=False,
         supported_extensions=None,
         exclude=None,
+        include=None,
     ):
         """
         Configure the reporter to use `git_diff` as the wrapper
         for the `git diff` tool.  (Should have same interface
         as `git_diff.GitDiffTool`)
         """
-        options = list()
+        options = []
         if not ignore_staged:
             options.append("staged")
         if not ignore_unstaged:
             options.append("unstaged")
+        if include_untracked:
+            options.append("untracked")
 
         # Branch is always present, so use as basis for name
-        name = "{}...HEAD".format(compare_branch)
+        name = f"{compare_branch}...HEAD"
         if len(options) > 0:
             # If more options are present separate them by comma's, except the last one
             for item in options[:-1]:
@@ -124,12 +138,13 @@ class GitDiffReporter(BaseDiffReporter):
             # Apply and + changes to the last option
             name += " and " + options[-1] + " changes"
 
-        super().__init__(name, exclude)
+        super().__init__(name, exclude, include)
 
         self._compare_branch = compare_branch
         self._git_diff_tool = git_diff
         self._ignore_staged = ignore_staged
         self._ignore_unstaged = ignore_unstaged
+        self._include_untracked = include_untracked
         self._supported_extensions = supported_extensions
 
         # Cache diff information as a dictionary
@@ -150,6 +165,13 @@ class GitDiffReporter(BaseDiffReporter):
         # Get the diff dictionary
         diff_dict = self._git_diff()
 
+        # include untracked files
+        if self._include_untracked:
+            for path in self._git_diff_tool.untracked():
+                with open(path) as file_handle:
+                    num_lines = len(file_handle.readlines())
+                diff_dict[path] = list(range(1, num_lines + 1))
+
         # Return the changed file paths (dict keys)
         # in alphabetical order
         return sorted(diff_dict.keys(), key=lambda x: x.lower())
@@ -195,31 +217,29 @@ class GitDiffReporter(BaseDiffReporter):
         # If we do not have a cached result, execute `git diff`
         if self._diff_dict is None:
 
-            result_dict = dict()
+            result_dict = {}
 
             for diff_str in self._get_included_diff_results():
                 # Parse the output of the diff string
                 diff_dict = self._parse_diff_str(diff_str)
 
-                for src_path in diff_dict.keys():
+                for src_path, (added_lines, deleted_lines) in diff_dict.items():
                     if self._is_path_excluded(src_path):
                         continue
                     # If no _supported_extensions provided, or extension present: process
-                    root, extension = os.path.splitext(src_path)
+                    _, extension = os.path.splitext(src_path)
                     extension = extension[1:].lower()
                     # 'not self._supported_extensions' tests for both None and empty list []
                     if (
                         not self._supported_extensions
                         or extension in self._supported_extensions
                     ):
-                        added_lines, deleted_lines = diff_dict[src_path]
-
                         # Remove any lines from the dict that have been deleted
                         # Include any lines that have been added
                         result_dict[src_path] = [
                             line
                             for line in result_dict.get(src_path, [])
-                            if not line in deleted_lines
+                            if line not in deleted_lines
                         ] + added_lines
 
             # Eliminate repeats and order line numbers
@@ -233,8 +253,8 @@ class GitDiffReporter(BaseDiffReporter):
         return self._diff_dict
 
     # Regular expressions used to parse the diff output
-    SRC_FILE_RE = re.compile(r'^diff --git "?a/.*"? "?b/([^ \n"]*)"?')
-    MERGE_CONFLICT_RE = re.compile(r"^diff --cc ([^ \n]*)")
+    SRC_FILE_RE = re.compile(r'^diff --git "?a/.*"? "?b/([^\n"]*)"?')
+    MERGE_CONFLICT_RE = re.compile(r"^diff --cc ([^\n]*)")
     HUNK_LINE_RE = re.compile(r"\+([0-9]*)")
 
     def _parse_diff_str(self, diff_str):
@@ -250,7 +270,7 @@ class GitDiffReporter(BaseDiffReporter):
         """
 
         # Create a dict to hold results
-        diff_dict = dict()
+        diff_dict = {}
 
         # Parse the diff string into sections by source file
         sections_dict = self._parse_source_sections(diff_str)
@@ -274,7 +294,7 @@ class GitDiffReporter(BaseDiffReporter):
         """
 
         # Create a dict to map source files to lines in the diff output
-        source_dict = dict()
+        source_dict = {}
 
         # Keep track of the current source file
         src_path = None
@@ -319,7 +339,7 @@ class GitDiffReporter(BaseDiffReporter):
                         # We tolerate other information before we have
                         # a source file defined, unless it's a hunk line
                         if line.startswith("@@"):
-                            msg = "Hunk has no source file: '{}'".format(line)
+                            msg = f"Hunk has no source file: '{line}'"
                             raise GitDiffError(msg)
 
         return source_dict
@@ -389,8 +409,6 @@ class GitDiffReporter(BaseDiffReporter):
                     current_line_new += 1
 
                 # If we are not in a hunk, then ignore the line
-                else:
-                    pass
 
         return added_lines, deleted_lines
 
@@ -404,7 +422,7 @@ class GitDiffReporter(BaseDiffReporter):
         elif "--cc" in line:
             regex = self.MERGE_CONFLICT_RE
         else:
-            msg = "Do not recognize format of source in line '{}'".format(line)
+            msg = f"Do not recognize format of source in line '{line}'"
             raise GitDiffError(msg)
 
         # Parse for the source file path
@@ -413,9 +431,8 @@ class GitDiffReporter(BaseDiffReporter):
         if len(groups) == 1:
             return groups[0]
 
-        else:
-            msg = "Could not parse source path in line '{}'".format(line)
-            raise GitDiffError(msg)
+        msg = f"Could not parse source path in line '{line}'"
+        raise GitDiffError(msg)
 
     def _parse_hunk_line(self, line):
         """
@@ -455,11 +472,11 @@ class GitDiffReporter(BaseDiffReporter):
                     raise GitDiffError(msg)
 
             else:
-                msg = "Could not find start of hunk in line '{}'".format(line)
+                msg = f"Could not find start of hunk in line '{line}'"
                 raise GitDiffError(msg)
 
         else:
-            msg = "Could not parse hunk in line '{}'".format(line)
+            msg = f"Could not parse hunk in line '{line}'"
             raise GitDiffError(msg)
 
     @staticmethod
@@ -476,4 +493,4 @@ class GitDiffReporter(BaseDiffReporter):
         line_set = set(line_numbers)
 
         # Retrieve the list from the set, sort it, and return
-        return sorted([line for line in line_set])
+        return sorted(line for line in line_set)
